=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed Dec. 2008 by Fredo6

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:  Lib6Mark.rb
# Original Date	:  23 Dec 2008 - version 3.0
# Type			:  Script library part of the LibFredo6 shared libraries
# Description	:  Contains some utilities for drawing standard predefined marks
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module G6

#--------------------------------------------------------------------------------------------------------------
# Class DrawMark: Predefined marks for 2D and 3D drawing 
#--------------------------------------------------------------------------------------------------------------			 

class AllDrawMark

Traductor_DrawMark_Elt = Struct.new "Traductor_DrawMark_Elt", :lpt, :color, :width, :stipple, :gl_type

def creation_mark
	@list_elts = []
	@stipple = ""
	@color = 'black'
	@width = 1
end

def draw_at_point2d(view, pt2d)
	size = (@size) ? @size : 1
	t = Geom::Transformation.translation(pt2d) * Geom::Transformation.scaling(size)

	@list_elts.each do |elt|
		lpt = elt.lpt
		lpt2d = lpt.collect { |pt| t * pt }
		view.drawing_color = (elt.color) ? elt.color : @color
		view.line_width = (elt.width) ? elt.width : @width
		view.line_stipple = (elt.stipple) ? elt.stipple : @stipple
		view.draw2d elt.gl_type, lpt2d
	end	
end

def draw_at_xy(view, x, y)
	draw_at_point2d view, Geom::Point3d.new(x, y, 0)
end

def draw_at_point3d(view, pt3d)
	draw_at_point2d view, view.screen_coords(pt3d)
end

def create_element(lpt, gl_type, color=nil, width=nil, stipple=nil)
	elt = Traductor_DrawMark_Elt.new
	elt.lpt = lpt
	elt.gl_type = gl_type
	elt.color = color
	elt.width = width
	elt.stipple = stipple
	elt
end

def create_line(lpt)
	create_element lpt, GL_LINE_STRIP
end

end	#class AllDrawMark

#--------------------------------------------------------------------------------------------------------------
# Class DrawMark_Forbidden: Forbidden sign 
#--------------------------------------------------------------------------------------------------------------			 

class DrawMark_Forbidden < AllDrawMark

def initialize(size=nil, color=nil, width=nil)
	creation_mark
	@size = (size) ? size : 5
	@color = (color) ? color : 'red'
	@width = (width) ? width : 2
	
	#Main circle
	n = 12
	pi = Math::PI
	radius = 10
	circle = []
	for i in 0..n
		angle = 2 * pi * i / n
		cosinus = Math::cos angle
		sinus = Math::sin angle
		circle.push Geom::Point3d.new(cosinus, sinus, 0)
	end
	bar = [circle[5], circle[11]]
	@list_elts.push create_line(circle)
	@list_elts.push create_line(bar)
end

end	#class DrawMark_Forbidden

#--------------------------------------------------------------------------------------------------------------
# Class DrawMark_FourArrows: 4 arrows 
#--------------------------------------------------------------------------------------------------------------			 

class DrawMark_FourArrows < AllDrawMark

def initialize(size=nil, color=nil, width=nil)
	creation_mark
	@size = (size) ? size : 7
	@color = (color) ? color : 'blue'
	@width = (width) ? width : 2
	
	#Shape
	dec = 0.4
	pt1 = Geom::Point3d.new -1, 0, 0
	pt2 = Geom::Point3d.new 1, 0, 0
	pt3 = Geom::Point3d.new 0, -1, 0
	pt4 = Geom::Point3d.new 0, 1, 0
	@list_elts.push create_line([pt1, pt2])
	@list_elts.push create_line([pt3, pt4])
	@list_elts += make_arrow(pt1, -dec, 'X')
	@list_elts += make_arrow(pt2, dec, 'X')
	@list_elts += make_arrow(pt3, -dec, 'Y')
	@list_elts += make_arrow(pt4, dec, 'Y')
end

def make_arrow(pt2d, dec, code)
	lselt = []
	case code
	when 'X'
		x1 = pt2d.x - dec
		y1 = pt2d.y + dec
		y2 = pt2d.y - dec
		lselt.push create_line([pt2d, Geom::Point3d.new(x1, y1, 0)])
		lselt.push create_line([pt2d, Geom::Point3d.new(x1, y2, 0)])
	when 'Y'
		y1 = pt2d.y - dec
		x1 = pt2d.x + dec
		x2 = pt2d.x - dec
		lselt.push create_line([pt2d, Geom::Point3d.new(x1, y1, 0)])
		lselt.push create_line([pt2d, Geom::Point3d.new(x2, y1, 0)])
	end
	lselt	
end

end	#class DrawMark_Forbidden


end #Module G6

